#include <avr/io.h>
#include <util/delay.h>
#include "MMC_SD/MMC_SD.h"
#include "FAT/FAT.h"

// control lines
#define	LCD_RESET	PC2
#define	LCD_RD    PC5
#define	LCD_WR    PC6
#define	LCD_RS	  PC7   
#define	LCD_CS	  PC4
#define LED_1     PD1
#define LED_2     PD2

// macros for control lines
#define CLR_RS PORTC &= ~(1<<LCD_RS);
#define SET_RS PORTC |=  (1<<LCD_RS);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RESET);
#define SET_RESET PORTC |=  (1<<LCD_RESET);

#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

#define LED1_OFF PORTD &= ~(1 << LED_1);
#define LED1_ON  PORTD |=  (1 << LED_1);
#define LED2_OFF PORTD &= ~(1 << LED_2);
#define LED2_ON  PORTD |=  (1 << LED_2);

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

// ************************************************************************* //
void write_command(unsigned char command)
{
	CLR_CS;       // chip selected
	CLR_RS;       // set to write to the control registers
	SET_RD;       // not reading
	SET_WR;       // not writing yet

	PORTA=0x00;   // write the command to the port

	CLR_WR;       // bring write line low
	SET_WR;       // clock in the command

	PORTA=command;// write the command to the port

	CLR_WR;       // bring write line low
	SET_WR;       // clock in the command

	SET_CS;       // unselect chip
	SET_RS;       // set for data
}
// ************************************************************************* //
void write_data(unsigned char data_h, unsigned char data_l)
{
	CLR_CS;       // chip selected
	SET_RS;       // set to write data
	SET_RD;       // not reading
	SET_WR;       // not writing yet

	PORTA=data_h; // write the command to the port

	CLR_WR;       // bring write line low
	SET_WR;       // clock in the command

	PORTA=data_l; // write the command to the port

	CLR_WR;       // bring write line low
	SET_WR;       // clock in the command

	SET_CS;       // unselect chip
	SET_RS;       // set for data
}
// ************************************************************************* //
void display_color(unsigned char data_h, unsigned char data_l)
{
	unsigned int i, j;
	write_command(0x20);
	write_data(0x00,0x00);
	write_command(0x21);
	write_data(0x00,0x00);
	write_command(0x22);

	for(i=0;i<220;i++)
	{
		for(j=0;j<176;j++)
		{
			write_data(data_h,data_l);
		}
	}
}
// ************************************************************************* //
void initialize_display()
{
   SET_RESET;
   _delay_ms(10);
   CLR_RESET;
   _delay_ms(100);
   SET_RESET;
   _delay_ms(500);

	//************* Start Initial Sequence **********// 
	write_command(0x01);			// Driver Output Control Register (P14, OTM2201A Datasheet)
	write_data(0x01,0x1C);			// Decrement Address counter
	// 528x220 dots (176xRGBx220)

	write_command(0x02);
	write_data(0x01,0x00);

	write_command(0x03);
	write_data(0x10,0x30);

	write_command(0x08);    
	write_data(0x08,0x08);

	write_command(0x0c);  
	write_data(0x00,0x00);

	write_command(0x0f); 
	write_data(0x00,0x01);

	write_command(0x20);
	write_data(0x00,0x00);

	write_command(0x21);
	write_data(0x00,0x00);

	//*************Power On sequence ****************// 
	write_command(0x10); 
	write_data(0x00,0x00);

	write_command(0x11);  
	write_data(0x10,0x00);
	_delay_ms(100);
	
	//------------------------ Set GRAM area --------------------------------// 
	write_command(0x30);
	write_data(0x00,0x00);

	write_command(0x31);
	write_data(0x00,0xdb);

	write_command(0x32);
	write_data(0x00,0x00);

	write_command(0x33);
	write_data(0x00,0x00);

	write_command(0x34);
	write_data(0x00,0xdb);

	write_command(0x35);
	write_data(0x00,0x00);

	write_command(0x36);
	write_data(0x00,0xaf);

	write_command(0x37);
	write_data(0x00,0x00);

	write_command(0x38);
	write_data(0x00,0xdb);

	write_command(0x39);
	write_data(0x00,0x00);
	_delay_ms(10);
	write_command(0xff);
	write_data(0x00,0x03);

	// ----------- Adjust the Gamma  Curve ----------// 
	write_command(0x50);
	write_data(0x02,0x03);

	write_command(0x051);
	write_data(0x0a,0x09);

	write_command(0x52);
	write_data(0x00,0x05);

	write_command(0x53);
	write_data(0x10,0x21);

	write_command(0x54);
	write_data(0x06,0x02);

	write_command(0x55);
	write_data(0x00,0x03);

	write_command(0x56);
	write_data(0x07,0x03);

	write_command(0x57);
	write_data(0x05,0x07);

	write_command(0x58);
	write_data(0x10,0x21);

	write_command(0x59);
	write_data(0x07,0x03);

	//***************************************
	write_command(0xB0); //VCOM 
	write_data(0x25,0x01); //??????????

	//********************************************************
	write_command(0xFF);
	write_data(0x00,0x00);

	write_command(0x07);
	write_data(0x10,0x17);
	_delay_ms(200);
	write_command(0x22);
}
// ************************************************************************* //
void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 slide_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
		return;

	buffer = malloc(512);
	if (buffer == 0)
		return;

next: 	//label for "goto"

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector

	pixels = (uint32) 176 * 220; // total # of pixels to write

	write_command(0x20);
	write_data(0x00,0x00);
	write_command(0x21);
	write_data(0x00,0x00);
	write_command(0x22);

	j = 0; // byte count

	while (1)
	{
		write_data(buffer[j]>>8,buffer[j]);  // write 16 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
			break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (slide_show_flag)
	{
		for (i=0;i<100;i++)	// delay for a while
			_delay_ms(50);

		pics++;					// inc pic number
		if (pics > totalfiles)	// if last
			pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
// ************************************************************************* //
int main()
{
	DDRC=0xFF;    // all pins as outputs
	DDRA=0xFF;    // all pins as outputs
	DDRD |= 0x06;	// Switches as inputs, LEDs as outputs

	PORTA=0x00;   // data lines low
	PORTC=0xFF;   // control and other lines high
	PORTD=0xF0;   // pull ups on the keys, remaining pins low

	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	_delay_ms(100);

	MMC_SD_Init();    //SPI initialize

	// Initialize the display
	initialize_display();

sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	OSCCAL = 0xFF;      //run at high speed for normal operation for SD Card

	_delay_ms(100);

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	Picture_Slide_Show();

	while (1)
	{
		;
	}
	return 0;
}
